@@ -5,7 +5,7 @@ use warnings;
 use utf8;
 use 5.10.0;
 
-use JSON::XS;
+use Mojo::JSON;
 use Digest::MD5                     qw(md5_hex);
 use Encode                          qw(encode_utf8);
 
@@ -23,6 +23,7 @@ Parse address from string
 sub parse {
     my ($class, $str) = @_;
 
+    return undef unless defined $str;
     my ($full, $address, $lon, $lat, $md5, $id, $type, $lang, $opt);
 
     if( $str =~ m{^\s*\[} and $str =~ m{\]\s*$} ) {
@@ -33,7 +34,13 @@ sub parse {
         eval { utf8::encode $str }    if utf8::is_utf8 $str;
 
         # Try parse
-        my $json = eval{ JSON::XS->new->utf8(1)->decode( $str ); };
+        my $json = eval{
+            if( version->new($Mojolicious::VERSION) < version->new(5.54) ) {
+                return Mojo::JSON->new->decode( $str );
+            } else {
+                return Mojo::JSON::decode_json( $str );
+            }
+        };
         if( not $@ and $json and 'ARRAY' eq ref($json)) {
             $full       = sprintf '%s : %s , %s',
                             $json->[2]//'', $json->[3]//'', $json->[4]//'';
